#Code 9.3 Ordinary Least Squares (OLS) Regression

rm(list=ls(all=TRUE))  #remove objects between runs

#Directories
dir.udf<-as.character("~/Documents/BOLOGNA/Didattica/NLTS/pheno/UserDefinedFunctions")

#Load user defined functions
setwd(dir.udf)
dump("lorenz_udf", file="lorenz_udf.R")
source("lorenz_udf.R")
dump("finite_differences_udf", file="finite_differences_udf.R")
source("finite_differences_udf.R")
dump("polynomial_udf", file="polynomial_udf.R")
source("polynomial_udf.R")

#Generate Lorenz data with user-defined function 'lorenz_udf'
lorenz_results<-lorenz_udf(x0=1,y0=1,z0=1,sigma=10,beta=8/3,rho=28,
 t.end=50,delta=0.001)
x<-lorenz_results[[1]];y<-lorenz_results[[2]];z<-lorenz_results[[3]]

#Set parameters for finite differences and polynomial expansion
delta<-0.001  #integration step
m<-2  #number of equations
Mp<-2  #polynomial range

#Response Variable:  User-defined function to compute finite differences
diff_x<-diff(x,delta)  #run user-defined function for x
diff_y<-diff(y,delta)
diff_z<-diff(z,delta)
x<-diff_x[[1]];y<-diff_y[[1]];z<-diff_z[[1]]  #times series same length differences
x.diff<-diff_x[[2]] #finite differences

#Regressors: User-defined function to compute multivariate polynomial expansion
poly.exp<-poly(m,Mp)
#Data Check: compare order of terms in 'mpoly' with column names
termList2<-poly.exp[[1]]  #monomial terms
poly1<-mpoly(termList2);term<-terms(poly1);term
col.names<-poly.exp[[2]];col.names  
#Monomials populated with data
data1<-poly.exp[[3]]  #populated data matrix
data2<-data1[complete.cases(data1),]  #remove last row with na's
data<-data2[,-1] #remove 1st column of 1's (regression packages don't require)

#User-defined function to run OLS
ols_udf<-function(dep,exog,m,Mp){
data.reg<-cbind(dep,exog);data.reg<-as.data.frame(data.reg) #create data frame
if(m==2) {
  if(Mp==2){
    eqn <- dep~x+y+xy
  }  #end Mp=2

  if(Mp==3){
    eqn <- dep~x+y+xy+x2+y2+x2y+xy2+x2y2
  }  #end Mp=3

  if(Mp==4) {
    eqn <- dep~x+y+xy+x2+y2+x2y+xy2+x2y2+x3+y3+x3y+xy3+x2y3+x3y2+x3y3
  }  #end Mp=4
} #end if m=2

if(m==3) {
  if(Mp==2){
    eqn <- dep~z+y+yz+x+xz+xy+xyz    
  }  #end Mp=2

  if(Mp==3){
    eqn <- dep~z+z2+y+yz+yz2+y2+y2z+y2z2+x+xz+xz2+xy+xyz+xyz2+xy2+xy2z+xy2z2+x2+x2z+x2z2+
      x2y+x2yz+x2yz2+x2y2+x2y2z+x2y2z2    
  }  #end Mp=3

  if(Mp==4) {
    eqn <- dep~z+z2+z3+y+yz+yz2+yz3+y2+y2z+y2z2+y2z3+y3+y3z+y3z2+y3z3+x+xz+xz2+xz3+
      xy+xyz+xyz2+xyz3+xy2+xy2z+xy2z2+xy2z3+xy3+xy3z+xy3z2+xy3z3+x2+x2z+x2z2+x2z3+x2y+
      x2yz+x2yz2+x2yz3+x2y2+x2y2z+x2y2z2+x2y2z3+x2y3+x2y3z+x2y3z2+x2y3z3+x3+x3z+x3z2+
      x3z3+x3y+x3yz+x3yz2+x3yz3+x3y2+x3y2z+x3y2z2+x3y2z3+x3y3+x3y3z+x3y3z2+x3y3z3 
  }  #end Mp=4
} #end if m=3

if(m==4) {
  if(Mp==2){
    eqn <- dep~w+z+zw+y+yw+yz+yzw+x+xw+xz+xzw+xy+xyw+xyz+xyzw
  }  #end if Mp=2

  if(Mp==3){
    eqn <- dep~w+w2+z+zw+zw2+z2+z2w+z2w2+y+yw+yw2+yz+yzw+yzw2+yz2+yz2w+yz2w2+y2+y2w+
      y2w2+y2z+y2zw+y2zw2+y2z2+y2z2w+y2z2w2+x+xw+xw2+xz+xzw+xzw2+xz2+xz2w+xz2w2+xy+xyw+
      xyw2+xyz+xyzw+xyzw2+xyz2+xyz2w+xyz2w2+xy2+xy2w+xy2w2+xy2z+xy2zw+xy2zw2+xy2z2+xy2z2w+
      xy2z2w2+x2+x2w+x2w2+x2z+x2zw+x2zw2+x2z2+x2z2w+x2z2w2+x2y+x2yw+x2yw2+x2yz+x2yzw+x2yzw2+
      x2yz2+x2yz2w+x2yz2w2+x2y2+x2y2w+x2y2w2+x2y2z+x2y2zw+x2y2zw2+x2y2z2+x2y2z2w+x2y2z2w2
  }  #end if Mp = 3
}  #end if m=4

if(m==5) {
  eqn <- dep~v+w+wv+z+zv+zw+zwv+y+yv+yw+ywv+yz+yzv+yzw+yzwv+x+xv+xw+xwv+xz+xzv+xzw+xzwv+
    xy+xyv+xyw+xywv+xyz+xyzv+xyzw+xyzwv
}  #end if m=5

fit<-lm(eqn,data.reg)
coeffs<-as.numeric(coef(fit))
table<-summary(fit)

results<-list(coeffs,table)
return(results)
} #end user defined function

results<-ols_udf(x.diff,data,m,Mp)
coeffs_ols<-results[[1]]
table<-results[[2]]
